

//PH - This file adapts Bigpicture by Joseph Ernest. See pinch-zoom JS file in this folder for full credits. Used under MIT license.
// Comments marked PH indicate Peter Hebden's adaptation of the code


/*
 *
 * bigpicture.js
 *
 * bigpicture.js is a library that allows infinite panning and infinite zooming in HTML pages.
 *               See it in action on http://www.bigpicture.bi/demo !
 *
 * author:  Joseph Ernest (twitter: @JosephErnest)
 * url:     http://github.com/josephernest/bigpicture.js
 *
 */


// PH - This version uses Hammer.js to handle pinch on touchscreen


var bigpicture = (function() {
  "use strict";

  /*
   * INITIALIZATION
   */

  // PH - adding variable to control zoom speed
  var zoom_in_speed = 1.5;
  var zoom_out_speed = 1.5;
  var delta_val;

  var bpContainer = document.getElementById('bigpicture-container'),
    bp = document.getElementById('bigpicture');

  if (!bp) { return; }

  bp.setAttribute('spellcheck', false);

  var params = { x: getQueryVariable('x'), y: getQueryVariable('y'), zoom: getQueryVariable('zoom') };

  var current = {};
  current.x = params.x ? parseFloat(params.x) : $(bp).data('x');
  current.y = params.y ? parseFloat(params.y) : $(bp).data('y');
  current.zoom = params.zoom ? parseFloat(params.zoom) : $(bp).data('zoom');

  //PH - SAVING ORIGINAL POSITION
  var original = {};
  original.x = current.x;
  original.y = current.y;
  original.zoom = 1;
  

  bp.x = 0; bp.y = 0;
  bp.updateContainerPosition = function() { bp.style.left = bp.x + 'px'; bp.style.top = bp.y + 'px'; };

  /*
   * TEXT BOXES
   */

  // PH - modifying initialisation to update x and y data on each element (sets font sizes, then data values based on div size, then positions it)
  $(".text").each(function() { 
    $(".text").each(function(){
      this.style.fontSize = $(this).data("size") / current.zoom + 'px';
    });
    $(this).data("x", ($(this).width() / 2));
    $(this).data("y", ($(this).height() / 2));
    updateTextPosition(this);
  });     // initialization

  $(bp).on('blur', '.text', function() { if ($(this).text().replace(/^\s+|\s+$/g, '') === '') { $(this).remove(); } });

  $(bp).on('input', '.text', function() { redoSearch = true; });

  /*
  function updateTextPosition(e) {
    e.style.fontSize = $(e).data("size") / current.zoom + 'px';
    e.style.left = ($(e).data("x") - current.x) / current.zoom - bp.x + 'px';
    e.style.top = ($(e).data("y") - current.y) / current.zoom - bp.y + 'px';
  }
  */

  // PH - modifying text position function to keep everything dead-centre
  // - x and y data values are now relative to the centre of the screen

  function updateTextPosition(e) {
    e.style.fontSize = $(e).data("size") / current.zoom + 'px';

    if (parseFloat(e.style.fontSize) > 2400 ) {
      e.style.opacity = "0";
    } else {
      e.style.opacity = "1";
    }

    // PH - stability control on very small objects - still not very accurate, but some improvement.
    if ($(e).data("x") == 1 && e.clientWidth > 2) {
      $(e).data("x", ($(e).width() / 2));
      $(e).data("y", ($(e).height() / 2));
    }

    // PH - zoom controls to be used in tandem with negative CSS margins on some large and small elements to adjust position
    e.style.left = ( window.innerWidth/2 - ($(e).data("x") / current.zoom) ) + 'px';
    e.style.top = ( window.innerHeight/2 - ($(e).data("y") / current.zoom) ) + 'px';


  }


  function newText(x, y, size, text) {
    var tb = document.createElement('div');
    tb.className = "text";
    tb.contentEditable = true;
    tb.innerHTML = text;
    $(tb).data("x", x).data("y", y).data("size", size);
    updateTextPosition(tb);
    bp.appendChild(tb);
    return tb;
  }

  /*
  // PH - disabling this method to stop new text boxes

  bpContainer.onclick = function(e) {
    if (isContainedByClass(e.target, 'text')) { return; }
    newText(current.x + (e.clientX) * current.zoom, current.y + (e.clientY) * current.zoom, 20 * current.zoom, '').focus();
  };
  /*

  /*
   * PAN AND MOVE
   */

   // PH - disabling all PAN AND MOVE functions to avoid confusion

    /*
  var movingText = null,
    dragging = false,
    previousMousePosition;

  bpContainer.onmousedown = function(e) {
    if ($(e.target).hasClass('text') && (e.ctrlKey || e.metaKey)) {
      movingText = e.target;
      movingText.className = "text noselect notransition";
    }
    else {
      movingText = null;
      dragging = true;
    }
    biggestPictureSeen = false;
    previousMousePosition = { x: e.pageX, y: e.pageY };
  };

  window.onmouseup = function() {
    dragging = false;
    if (movingText) { movingText.className = "text"; }
    movingText = null;
  };

  bpContainer.ondragstart = function(e) {
    e.preventDefault();
  };

  bpContainer.onmousemove = function(e) {
    if (dragging && !e.shiftKey) {       // SHIFT prevents panning / allows selection
      bp.style.transitionDuration = "0s";
      bp.x += e.pageX - previousMousePosition.x;
      bp.y += e.pageY - previousMousePosition.y;
      bp.updateContainerPosition();
      current.x -= (e.pageX - previousMousePosition.x) * current.zoom;
      current.y -= (e.pageY - previousMousePosition.y) * current.zoom;
      previousMousePosition = { x: e.pageX, y: e.pageY };
    }
    if (movingText) {
      $(movingText).data("x", $(movingText).data("x") + (e.pageX - previousMousePosition.x) * current.zoom);
      $(movingText).data("y", $(movingText).data("y") + (e.pageY - previousMousePosition.y) * current.zoom);
      updateTextPosition(movingText);
      previousMousePosition = { x: e.pageX, y: e.pageY };
    }
  };

  */



  
  /*
   * ZOOM
   */

  bpContainer.ondblclick = function(e) {
    e.preventDefault();
    onZoom((e.ctrlKey || e.metaKey) ? current.zoom * 1.7 * 1.7 : current.zoom / 1.7 / 1.7, current.x + e.clientX * current.zoom, current.y + e.clientY * current.zoom, e.clientX, e.clientY);
  };

  var biggestPictureSeen = false,
    previous;

  function onZoom(zoom, wx, wy, sx, sy) {  // zoom on (wx, wy) (world coordinates) which will be placed on (sx, sy) (screen coordinates)
    //wx = (typeof wx === "undefined") ? current.x + window.innerWidth / 2 * current.zoom : wx;
    //wy = (typeof wy === "undefined") ? current.y + window.innerHeight / 2 * current.zoom : wy;
    //sx = (typeof sx === "undefined") ? window.innerWidth / 2  : sx;
    //sy = (typeof sy === "undefined") ? window.innerHeight / 2 : sy;

    //PH - always zoom on screen centre
    wx = current.x + window.innerWidth / 2 * current.zoom;
    wy = current.y + window.innerHeight / 2 * current.zoom;
    sx = window.innerWidth / 2;
    sy = window.innerHeight / 2;

    // PH - transition effect disabled
    bp.style.transitionDuration = "0.2s";

    bp.x = 0; bp.y = 0;
    bp.updateContainerPosition();
    current.x = wx - sx * zoom; current.y = wy - sy * zoom; current.zoom = zoom;

    $(".text").each(function() { updateTextPosition(this); });

    biggestPictureSeen = false;

    //PH - LOGGING CURRENT ZOOM LEVEL
    console.log("Zoom: "+current.zoom);
    $("#debug_scale").html(current.zoom);

    //PH - changing appearance based on zoom level
    change_appearance(current.zoom);

  }

  function zoomOnText(res) {
    onZoom($(res).data('size') / 20, $(res).data('x'), $(res).data('y'));
  }

  function seeBiggestPicture(e) {
    e.preventDefault();
    document.activeElement.blur();
    function universeboundingrect() {
      var minX = Infinity, maxX = - Infinity, minY = Infinity, maxY = - Infinity;
      var texteelements = document.getElementsByClassName('text');
      [].forEach.call(texteelements, function(elt) {
        var rect2 = elt.getBoundingClientRect();
        var rect = { left: $(elt).data("x"),
                     top: $(elt).data("y"),
                     right: (rect2.width > 2 && rect2.width < 10000) ? current.x + rect2.right * current.zoom : $(elt).data("x") + 300 * $(elt).data("size") / 20,
                     bottom: (rect2.height > 2 && rect2.height < 10000) ? current.y + rect2.bottom * current.zoom : $(elt).data("y") + 100 * $(elt).data("size") / 20 };
        if (rect.left < minX) { minX = rect.left; }
        if (rect.right > maxX) { maxX = rect.right; }
        if (rect.top < minY) { minY = rect.top; }
        if (rect.bottom > maxY) { maxY = rect.bottom; }
      });
      return { minX: minX, maxX: maxX, minY: minY, maxY: maxY };
    }

    var texts = document.getElementsByClassName('text');
    if (texts.length === 0) { return; } 
    if (texts.length === 1) { zoomOnText(texts[0]); return; }

    if (!biggestPictureSeen) {
      previous = { x: current.x, y: current.y, zoom: current.zoom };
      var rect = universeboundingrect();
      var zoom = Math.max((rect.maxX - rect.minX) / window.innerWidth, (rect.maxY - rect.minY) / window.innerHeight) * 1.1;
      onZoom(zoom, (rect.minX + rect.maxX) / 2, (rect.minY + rect.maxY) / 2);
      biggestPictureSeen = true;
    }
    else {
      onZoom(previous.zoom, previous.x, previous.y, 0, 0);
      biggestPictureSeen = false;
    }
  }


  // PH - ADDING A RESET FUNCTION
  function seeOriginalPicture(e) {
    e.preventDefault();
    document.activeElement.blur();
    var texts = document.getElementsByClassName('text');
    if (texts.length === 0) { return; } 
    if (texts.length === 1) { zoomOnText(texts[0]); return; }

    previous = { x: current.x, y: current.y, zoom: current.zoom };
    var zoom = 1;
    onZoom(zoom, (original.x + (window.innerWidth/2) ), (original.y + (window.innerHeight/2) ) );
    biggestPictureSeen = false;
    
  }

  //PH - ADDING A FUNCTION TO CHANGE APPEARANCE BASED ON ZOOM
  
  function change_appearance(z) {
    switch (true) {
      case (z < 0.1) :
        $("#bigpicture-container").css({
          "background-color":"green",
          "color":"white"
        });
        break;
      case (z > 5.6*(10**35) ) :
        $("#bigpicture-container").css({
          "background-color":"black",
          "color":"white"
        });
        break;
      default :
        $("#bigpicture-container").css({
          "background-color":"transparent",
          "color":"black"
        });
    }
  }
  

  /*
   * SEARCH
   */

  var results = { index: -1, elements: [], text: "" },
    redoSearch = true,
    query;

  function find(txt) {
    results = { index: -1, elements: [], text: txt };
    $(".text").each(function() {
      if ($(this).text().toLowerCase().indexOf(txt.toLowerCase()) != -1) { results.elements.push(this); }
    });
    if (results.elements.length > 0) { results.index = 0; }
  }

  function findNext(txt) {
    if (!txt || txt.replace(/^\s+|\s+$/g, '') === '') { return; }   // empty search
    if (results.index == -1 || results.text != txt || redoSearch) {
      find(txt);
      if (results.index == -1) { return; }       // still no results
      redoSearch = false;
    }
    var res = results.elements[results.index];
    zoomOnText(res);
    results.index += 1;
    if (results.index == results.elements.length) { results.index = 0; }  // loop
  }

  /*
   * MOUSEWHEEL
   */

  var mousewheeldelta = 0, 
    last_e, 
    mousewheeltimer = null, 
    mousewheel;

  if (navigator.appVersion.indexOf("Mac") != -1) {   // Mac OS X
    mousewheel = function(e) {
      e.preventDefault();
      mousewheeldelta += Math.max(-1, Math.min(1, (e.wheelDelta || -e.detail)));
      console.log("delta:"+mousewheeldelta);
      last_e = e;
      delta_val = mousewheeldelta;
      if (!mousewheeltimer) {
        mousewheeltimer = setTimeout(function() {
          //PH - if statement - zoom using mousewheel if...
          if ( 
            (current.zoom <= 1 && mousewheeldelta < 0) // past zoom-in limit but zooming out
            ||
            (current.zoom > 1 && current.zoom <= 10**37) //within zoom range
            || 
            (current.zoom > 10**37 && mousewheeldelta > 0) // past zoom-out limit but zooming in
          ) {
            onZoom((mousewheeldelta > 0) ? current.zoom / zoom_in_speed : current.zoom * zoom_out_speed, current.x + last_e.clientX * current.zoom, current.y + last_e.clientY * current.zoom, last_e.clientX, last_e.clientY);
          }
          mousewheeldelta = 0;
          mousewheeltimer = null;
          
        }, 70);
      }
    };
  }
  else {
    mousewheel = function(e) {
      e.preventDefault();
      var delta = Math.max(-1, Math.min(1, (e.wheelDelta || -e.detail)));
      delta_val = delta;
      //PH - if statement - zoom using mousewheel if...
      if ( 
        (current.zoom <= 1 && delta < 0) // past zoom-in limit but zooming out
        ||
        (current.zoom > 1 && current.zoom <= 10**37) //within zoom range
        || 
        (current.zoom > 10**37 && delta > 0) // past zoom-out limit but zooming in
      ) {
        onZoom((delta > 0) ? current.zoom / zoom_in_speed : current.zoom * zoom_out_speed, current.x + e.clientX * current.zoom, current.y + e.clientY * current.zoom, e.clientX, e.clientY);
      }
    };
  }

  if ("onmousewheel" in document) { bpContainer.onmousewheel = mousewheel; }
  else { bpContainer.addEventListener('DOMMouseScroll', mousewheel, false); }


  //PH - adding some Hammer to replicate the above with pinch-zoom - NEEDS TWEAKING

  var mc = new Hammer.Manager(bpContainer);

  // create a pinch recognizer
  // these require 2 pointers
  var pinch = new Hammer.Pinch();

  // add to the Manager
  mc.add(pinch);

  var old_scale = 0;
  mc.on("pinch", function(ev){
    ev.preventDefault();
    //var delta = Math.max(-1, Math.min(1, ev.scale));
    var delta = ev.scale - old_scale;
    $("#debug_scale").html(ev.scale + " , " + delta);
    delta_val = delta;
    //PH - if statement - zoom using mousewheel if...
    if ( 
      (current.zoom <= 1 && mousewheeldelta < 0) // past zoom-in limit but zooming out
      ||
      (current.zoom > 1 && current.zoom <= 10**37) //within zoom range
      || 
      (current.zoom > 10**37 && mousewheeldelta > 0) // past zoom-out limit but zooming in
    ) {
      // VALUES IN HERE NEED ADJUSTING?
      onZoom((delta > 0) ? current.zoom / zoom_in_speed : current.zoom * zoom_out_speed, current.x + ev.clientX * current.zoom, current.y + ev.clientY * current.zoom, ev.clientX, ev.clientY);
    }
    old_scale = ev.scale;
  });
  
  

  /*
   * KEYBOARD SHORTCUTS
   */

  window.onkeydown = function(e) {
    if (((e.ctrlKey && !e.altKey || e.metaKey) && (e.keyCode == 61 || e.keyCode == 187 || e.keyCode == 171 || e.keyCode == 107 || e.key == '+' || e.key == '=' ))   // CTRL+PLUS or COMMAND+PLUS 
    || e.keyCode == 34) {   // PAGE DOWN     // !e.altKey to prevent catching of ALT-GR 
      e.preventDefault();
      onZoom(current.zoom / 1.7);
      return;
    }
    if (((e.ctrlKey && !e.altKey || e.metaKey) && (e.keyCode == 54 || e.keyCode == 189 || e.keyCode == 173 || e.keyCode == 167 || e.keyCode == 109 || e.keyCode == 169 || e.keyCode == 219 || e.key == '-' ))   // CTRL+MINUS or COMMAND+MINUS
    || e.keyCode == 33) {   // PAGE UP
      e.preventDefault();
      onZoom(current.zoom * 1.7);
      return;
    }
    if ((e.ctrlKey && !e.altKey || e.metaKey) && e.keyCode == 70) {         // CTRL+F
      e.preventDefault();
      setTimeout(function() { query = window.prompt("What are you looking for?", ""); findNext(query); }, 10);
      return;
    }
    if (e.keyCode == 114) {                 // F3
      e.preventDefault();
      if (results.index == -1) { setTimeout(function() { query = window.prompt("What are you looking for?", ""); findNext(query); }, 10); }
      else { findNext(query); }
      return;
    }
    if (e.keyCode == 113) {                 // F2
      e.preventDefault();
      seeBiggestPicture(e);
      return;
    }

    if (e.keyCode == 32) {                 // PH - SPACE 
      console.log("space!")
      e.preventDefault();
      seeOriginalPicture(e);
      return;
    }
  };

  /*
   * USEFUL FUNCTIONS
   */

  function isContainedByClass(e, cls) { while (e && e.tagName) { if (e.classList.contains(cls)) { return true; } e = e.parentNode; } return false; }

  function getQueryVariable(id) { var params = window.location.search.substring(1).split("&");  for (var i = 0; i < params.length; i++) { var p = params[i].split("="); if (p[0] == id) { return p[1]; } } return(false); }

  /*
   * API
   */

  return { newText: newText, 
           current: current, 
           updateTextPosition: updateTextPosition };

})();